@@ -1,4 +1,6 @@
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python
+# coding: utf-8
+
 """
 Contains Python equivalents of the function and constant
 definitions in CANLIB's canlib.h, with some supporting functionality
@@ -7,6 +9,8 @@ specific to Python.
 Copyright (C) 2010 Dynamic Controls
 """
 
+from __future__ import absolute_import
+
 import sys
 import time
 import logging
@@ -14,7 +18,7 @@ import ctypes
 
 from can import CanError, BusABC
 from can import Message
-from can.interfaces.kvaser import constants as canstat
+from . import constants as canstat
 
 log = logging.getLogger('can.kvaser')
 
@@ -169,6 +173,12 @@ if __canlib is not None:
                                             restype=canstat.c_canStatus,
                                             errcheck=__check_status)
 
+    canSetBusParamsFd = __get_canlib_function("canSetBusParamsFd",
+                                            argtypes=[c_canHandle, ctypes.c_long,
+                                                      ctypes.c_uint, ctypes.c_uint,
+                                                      ctypes.c_uint],
+                                            restype=canstat.c_canStatus,
+                                            errcheck=__check_status)
 
     canSetBusOutputControl = __get_canlib_function("canSetBusOutputControl",
                                                    argtypes=[c_canHandle,
@@ -254,15 +264,25 @@ DRIVER_MODE_SILENT = False
 DRIVER_MODE_NORMAL = True
 
 
-BITRATE_OBJS = {1000000 : canstat.canBITRATE_1M,
-                 500000 : canstat.canBITRATE_500K,
-                 250000 : canstat.canBITRATE_250K,
-                 125000 : canstat.canBITRATE_125K,
-                 100000 : canstat.canBITRATE_100K,
-                  83000 : canstat.canBITRATE_83K,
-                  62000 : canstat.canBITRATE_62K,
-                  50000 : canstat.canBITRATE_50K,
-                  10000 : canstat.canBITRATE_10K}
+BITRATE_OBJS = {
+    1000000: canstat.canBITRATE_1M,
+    500000: canstat.canBITRATE_500K,
+    250000: canstat.canBITRATE_250K,
+    125000: canstat.canBITRATE_125K,
+    100000: canstat.canBITRATE_100K,
+    83000: canstat.canBITRATE_83K,
+    62000: canstat.canBITRATE_62K,
+    50000: canstat.canBITRATE_50K,
+    10000: canstat.canBITRATE_10K
+}
+
+BITRATE_FD = {
+    500000: canstat.canFD_BITRATE_500K_80P,
+    1000000: canstat.canFD_BITRATE_1M_80P,
+    2000000: canstat.canFD_BITRATE_2M_80P,
+    4000000: canstat.canFD_BITRATE_4M_80P,
+    8000000: canstat.canFD_BITRATE_8M_60P
+}
 
 
 class KvaserBus(BusABC):
@@ -276,15 +296,14 @@ class KvaserBus(BusABC):
             The Channel id to create this bus with.
 
         :param list can_filters:
-            A list of dictionaries each containing a "can_id" and a "can_mask".
-
-            >>> [{"can_id": 0x11, "can_mask": 0x21}]
-
+            See :meth:`can.BusABC.set_filters`.
 
         Backend Configuration
 
         :param int bitrate:
             Bitrate of channel in bit/s
+        :param bool accept_virtual:
+            If virtual channels should be accepted.
         :param int tseg1:
             Time segment 1, that is, the number of quanta from (but not including)
             the Sync Segment to the sampling point.
@@ -294,8 +313,8 @@ class KvaserBus(BusABC):
             Time segment 2, that is, the number of quanta from the sampling
             point to the end of the bit.
         :param int sjw:
-            The Synchronisation Jump Width. Decides the maximum number of time quanta
-            that the controller can resynchronise every bit.
+            The Synchronization Jump Width. Decides the maximum number of time quanta
+            that the controller can resynchronize every bit.
         :param int no_samp:
             Either 1 or 3. Some CAN controllers can also sample each bit three times.
             In this case, the bit will be sampled three quanta in a row,
@@ -313,7 +332,14 @@ class KvaserBus(BusABC):
             Only works if single_handle is also False.
             If you want to receive messages from other applications on the same
             computer, set this to True or set single_handle to True.
+        :param bool fd:
+            If CAN-FD frames should be supported.
+        :param int data_bitrate:
+            Which bitrate to use for data phase in CAN FD.
+            Defaults to arbitration bitrate.
+
         """
+
         log.info("CAN Filters: {}".format(can_filters))
         log.info("Got configuration of: {}".format(config))
         bitrate = config.get('bitrate', 500000)
@@ -324,6 +350,9 @@ class KvaserBus(BusABC):
         driver_mode = config.get('driver_mode', DRIVER_MODE_NORMAL)
         single_handle = config.get('single_handle', False)
         receive_own_messages = config.get('receive_own_messages', False)
+        accept_virtual = config.get('accept_virtual', True)
+        fd = config.get('fd', False)
+        data_bitrate = config.get('data_bitrate', None)
 
         try:
             channel = int(channel)
@@ -331,9 +360,6 @@ class KvaserBus(BusABC):
             raise ValueError('channel must be an integer')
         self.channel = channel
 
-        if 'tseg1' not in config and bitrate in BITRATE_OBJS:
-            bitrate = BITRATE_OBJS[bitrate]
-
         log.debug('Initialising bus instance')
         self.single_handle = single_handle
 
@@ -348,12 +374,33 @@ class KvaserBus(BusABC):
             if idx == channel:
                 self.channel_info = channel_info
 
+        flags = 0
+        if accept_virtual:
+            flags |= canstat.canOPEN_ACCEPT_VIRTUAL
+        if fd:
+            flags |= canstat.canOPEN_CAN_FD
+
         log.debug('Creating read handle to bus channel: %s' % channel)
-        self._read_handle = canOpenChannel(channel, canstat.canOPEN_ACCEPT_VIRTUAL)
+        self._read_handle = canOpenChannel(channel, flags)
         canIoCtl(self._read_handle,
                  canstat.canIOCTL_SET_TIMER_SCALE,
                  ctypes.byref(ctypes.c_long(TIMESTAMP_RESOLUTION)),
                  4)
+        
+        if fd:
+            if 'tseg1' not in config and bitrate in BITRATE_FD:
+                # Use predefined bitrate for arbitration
+                bitrate = BITRATE_FD[bitrate]
+            if data_bitrate in BITRATE_FD:
+                # Use predefined bitrate for data
+                data_bitrate = BITRATE_FD[data_bitrate]
+            elif not data_bitrate:
+                # Use same bitrate for arbitration and data phase
+                data_bitrate = bitrate
+            canSetBusParamsFd(self._read_handle, bitrate, tseg1, tseg2, sjw)
+        else:
+            if 'tseg1' not in config and bitrate in BITRATE_OBJS:
+                bitrate = BITRATE_OBJS[bitrate]
         canSetBusParams(self._read_handle, bitrate, tseg1, tseg2, sjw, no_samp, 0)
 
         # By default, use local echo if single handle is used (see #160)
@@ -370,11 +417,9 @@ class KvaserBus(BusABC):
             self._write_handle = self._read_handle
         else:
             log.debug('Creating separate handle for TX on channel: %s' % channel)
-            self._write_handle = canOpenChannel(channel, canstat.canOPEN_ACCEPT_VIRTUAL)
+            self._write_handle = canOpenChannel(channel, flags)
             canBusOn(self._read_handle)
 
-        self.set_filters(can_filters)
-
         can_driver_mode = canstat.canDRIVER_SILENT if driver_mode == DRIVER_MODE_SILENT else canstat.canDRIVER_NORMAL
         canSetBusOutputControl(self._write_handle, can_driver_mode)
         log.debug('Going bus on TX handle')
@@ -388,43 +433,33 @@ class KvaserBus(BusABC):
             log.info(str(exc))
         self._timestamp_offset = time.time() - (timer.value * TIMESTAMP_FACTOR)
 
-        super(KvaserBus, self).__init__()
+        self._is_filtered = False
+        super(KvaserBus, self).__init__(channel=channel, can_filters=can_filters, **config)
 
-    def set_filters(self, can_filters=None):
-        """Apply filtering to all messages received by this Bus.
-
-        Calling without passing any filters will reset the applied filters.
-
-        Since Kvaser only supports setting one filter per handle, the filtering
-        will be disabled if more than one filter is requested.
-
-        :param list can_filters:
-            A list of dictionaries each containing a "can_id", "can_mask" and
-            "extended".
-
-            >>> [{"can_id": 0x11, "can_mask": 0x21, "extended": False}]
-
-            A filter matches, when ``<received_can_id> & can_mask == can_id & can_mask``
-        """
-        if can_filters and len(can_filters) == 1:
-            can_id = can_filters[0]['can_id']
-            can_mask = can_filters[0]['can_mask']
-            extended = 1 if can_filters[0].get('extended') else 0
+    def _apply_filters(self, filters):
+        if filters and len(filters) == 1:
+            can_id = filters[0]['can_id']
+            can_mask = filters[0]['can_mask']
+            extended = 1 if filters[0].get('extended') else 0
             try:
                 for handle in (self._read_handle, self._write_handle):
                     canSetAcceptanceFilter(handle, can_id, can_mask, extended)
             except (NotImplementedError, CANLIBError) as e:
+                self._is_filtered = False
                 log.error('Filtering is not supported - %s', e)
             else:
+                self._is_filtered = True
                 log.info('canlib is filtering on ID 0x%X, mask 0x%X', can_id, can_mask)
 
         else:
+            self._is_filtered = False
             log.info('Hardware filtering has been disabled')
             try:
                 for handle in (self._read_handle, self._write_handle):
                     for extended in (0, 1):
                         canSetAcceptanceFilter(handle, 0, 0, extended)
             except (NotImplementedError, CANLIBError):
+                # TODO add logging?
                 pass
 
     def flush_tx_buffer(self):
@@ -432,12 +467,12 @@ class KvaserBus(BusABC):
         """
         canIoCtl(self._write_handle, canstat.canIOCTL_FLUSH_TX_BUFFER, 0, 0)
 
-    def recv(self, timeout=None):
+    def _recv_internal(self, timeout=None):
         """
-        Read a message from kvaser device.
+        Read a message from kvaser device and return whether filtering has taken place.
         """
         arb_id = ctypes.c_long(0)
-        data = ctypes.create_string_buffer(8)
+        data = ctypes.create_string_buffer(64)
         dlc = ctypes.c_uint(0)
         flags = ctypes.c_uint(0)
         timestamp = ctypes.c_ulong(0)
@@ -467,6 +502,9 @@ class KvaserBus(BusABC):
             is_extended = bool(flags & canstat.canMSG_EXT)
             is_remote_frame = bool(flags & canstat.canMSG_RTR)
             is_error_frame = bool(flags & canstat.canMSG_ERROR_FRAME)
+            is_fd = bool(flags & canstat.canFDMSG_FDF)
+            bitrate_switch = bool(flags & canstat.canFDMSG_BRS)
+            error_state_indicator = bool(flags & canstat.canFDMSG_ESI)
             msg_timestamp = timestamp.value * TIMESTAMP_FACTOR
             rx_msg = Message(arbitration_id=arb_id.value,
                              data=data_array[:dlc.value],
@@ -474,15 +512,18 @@ class KvaserBus(BusABC):
                              extended_id=is_extended,
                              is_error_frame=is_error_frame,
                              is_remote_frame=is_remote_frame,
+                             is_fd=is_fd,
+                             bitrate_switch=bitrate_switch,
+                             error_state_indicator=error_state_indicator,
                              channel=self.channel,
                              timestamp=msg_timestamp + self._timestamp_offset)
             rx_msg.flags = flags
             rx_msg.raw_timestamp = msg_timestamp
             #log.debug('Got message: %s' % rx_msg)
-            return rx_msg
+            return rx_msg, self._is_filtered
         else:
             #log.debug('read complete -> status not okay')
-            return None
+            return None, self._is_filtered
 
     def send(self, msg, timeout=None):
         #log.debug("Writing a message: {}".format(msg))
@@ -491,6 +532,10 @@ class KvaserBus(BusABC):
             flags |= canstat.canMSG_RTR
         if msg.is_error_frame:
             flags |= canstat.canMSG_ERROR_FRAME
+        if msg.is_fd:
+            flags |= canstat.canFDMSG_FDF
+        if msg.bitrate_switch:
+            flags |= canstat.canFDMSG_BRS
         ArrayConstructor = ctypes.c_byte * msg.dlc
         buf = ArrayConstructor(*msg.data)
         canWrite(self._write_handle,
@@ -520,15 +565,28 @@ class KvaserBus(BusABC):
         # Wait for transmit queue to be cleared
         try:
             canWriteSync(self._write_handle, 100)
-        except CANLIBError as e:
-            log.warning("There may be messages in the transmit queue that could "
-                        "not be transmitted before going bus off (%s)", e)
+        except CANLIBError:
+            # Not a huge deal and it seems that we get timeout if no messages
+            # exists in the buffer at all
+            pass
         if not self.single_handle:
             canBusOff(self._read_handle)
             canClose(self._read_handle)
         canBusOff(self._write_handle)
         canClose(self._write_handle)
 
+    @staticmethod
+    def _detect_available_configs():
+        num_channels = ctypes.c_int(0)
+        try:
+            canGetNumberOfChannels(ctypes.byref(num_channels))
+        except Exception:
+            pass
+        return [
+            {'interface': 'kvaser', 'channel': channel}
+            for channel in range(num_channels.value)
+        ]
+
 
 def get_channel_info(channel):
     name = ctypes.create_string_buffer(80)
