@@ -1,29 +1,23 @@
-#  PCANBasic.py
-#
-#  ~~~~~~~~~~~~
-#
-#  PCAN-Basic API
-#
-#  ~~~~~~~~~~~~
-#
-#  ------------------------------------------------------------------
-#  Author : Keneth Wagner
-#  Last change: 18.05.2016 Wagner
-#
-#  Language: Python 2.7
-#  ------------------------------------------------------------------
-#
-#  Copyright (C) 1999-2016  PEAK-System Technik GmbH, Darmstadt
-#  more Info at http://www.peak-system.com 
-#
+#!/usr/bin/env python
+# coding: utf-8
+
+"""
+PCAN-Basic API
+
+Author: Keneth Wagner
+Last change: 13.11.2017 Wagner
+
+Language: Python 2.7, 3.5
+
+Copyright (C) 1999-2017 PEAK-System Technik GmbH, Darmstadt, Germany
+http://www.peak-system.com
+"""
 
-# Module Imports
-#
 from ctypes import *
+from string import *
 import platform
 import logging
 
-# Patched for python-can: use logger instead of print()
 logger = logging.getLogger('can.pcan')
 
 #///////////////////////////////////////////////////////////
@@ -46,7 +40,7 @@ TPCANTimestampFD       = c_ulonglong # Represents a timestamp of a received PCAN
 #///////////////////////////////////////////////////////////
 
 # Currently defined and supported PCAN channels
-#
+
 PCAN_NONEBUS             = TPCANHandle(0x00)  # Undefined/default value for a PCAN bus
 
 PCAN_ISABUS1             = TPCANHandle(0x21)  # PCAN-ISA interface, channel 1
@@ -65,17 +59,17 @@ PCAN_PCIBUS2             = TPCANHandle(0x42)  # PCAN-PCI interface, channel 2
 PCAN_PCIBUS3             = TPCANHandle(0x43)  # PCAN-PCI interface, channel 3
 PCAN_PCIBUS4             = TPCANHandle(0x44)  # PCAN-PCI interface, channel 4
 PCAN_PCIBUS5             = TPCANHandle(0x45)  # PCAN-PCI interface, channel 5
-PCAN_PCIBUS6           = TPCANHandle(0x46)  # PCAN-PCI interface, channel 6
-PCAN_PCIBUS7           = TPCANHandle(0x47)  # PCAN-PCI interface, channel 7
-PCAN_PCIBUS8           = TPCANHandle(0x48)  # PCAN-PCI interface, channel 8
-PCAN_PCIBUS9           = TPCANHandle(0x409)  # PCAN-PCI interface, channel 9
-PCAN_PCIBUS10          = TPCANHandle(0x40A)  # PCAN-PCI interface, channel 10
-PCAN_PCIBUS11          = TPCANHandle(0x40B)  # PCAN-PCI interface, channel 11
-PCAN_PCIBUS12          = TPCANHandle(0x40C)  # PCAN-PCI interface, channel 12
-PCAN_PCIBUS13          = TPCANHandle(0x40D)  # PCAN-PCI interface, channel 13
-PCAN_PCIBUS14          = TPCANHandle(0x40E)  # PCAN-PCI interface, channel 14
-PCAN_PCIBUS15          = TPCANHandle(0x40F)  # PCAN-PCI interface, channel 15
-PCAN_PCIBUS16          = TPCANHandle(0x410)  # PCAN-PCI interface, channel 16
+PCAN_PCIBUS6             = TPCANHandle(0x46)  # PCAN-PCI interface, channel 6
+PCAN_PCIBUS7             = TPCANHandle(0x47)  # PCAN-PCI interface, channel 7
+PCAN_PCIBUS8             = TPCANHandle(0x48)  # PCAN-PCI interface, channel 8
+PCAN_PCIBUS9             = TPCANHandle(0x409) # PCAN-PCI interface, channel 9
+PCAN_PCIBUS10            = TPCANHandle(0x40A) # PCAN-PCI interface, channel 10
+PCAN_PCIBUS11            = TPCANHandle(0x40B) # PCAN-PCI interface, channel 11
+PCAN_PCIBUS12            = TPCANHandle(0x40C) # PCAN-PCI interface, channel 12
+PCAN_PCIBUS13            = TPCANHandle(0x40D) # PCAN-PCI interface, channel 13
+PCAN_PCIBUS14            = TPCANHandle(0x40E) # PCAN-PCI interface, channel 14
+PCAN_PCIBUS15            = TPCANHandle(0x40F) # PCAN-PCI interface, channel 15
+PCAN_PCIBUS16            = TPCANHandle(0x410) # PCAN-PCI interface, channel 16
 
 PCAN_USBBUS1             = TPCANHandle(0x51)  # PCAN-USB interface, channel 1
 PCAN_USBBUS2             = TPCANHandle(0x52)  # PCAN-USB interface, channel 2
@@ -85,38 +79,37 @@ PCAN_USBBUS5             = TPCANHandle(0x55)  # PCAN-USB interface, channel 5
 PCAN_USBBUS6             = TPCANHandle(0x56)  # PCAN-USB interface, channel 6
 PCAN_USBBUS7             = TPCANHandle(0x57)  # PCAN-USB interface, channel 7
 PCAN_USBBUS8             = TPCANHandle(0x58)  # PCAN-USB interface, channel 8
-PCAN_USBBUS9             = TPCANHandle(0x509)  # PCAN-USB interface, channel 9
-PCAN_USBBUS10            = TPCANHandle(0x50A)  # PCAN-USB interface, channel 10
-PCAN_USBBUS11            = TPCANHandle(0x50B)  # PCAN-USB interface, channel 11
-PCAN_USBBUS12            = TPCANHandle(0x50C)  # PCAN-USB interface, channel 12
-PCAN_USBBUS13            = TPCANHandle(0x50D)  # PCAN-USB interface, channel 13
-PCAN_USBBUS14            = TPCANHandle(0x50E)  # PCAN-USB interface, channel 14
-PCAN_USBBUS15            = TPCANHandle(0x50F)  # PCAN-USB interface, channel 15
-PCAN_USBBUS16            = TPCANHandle(0x510)  # PCAN-USB interface, channel 16
+PCAN_USBBUS9             = TPCANHandle(0x509) # PCAN-USB interface, channel 9
+PCAN_USBBUS10            = TPCANHandle(0x50A) # PCAN-USB interface, channel 10
+PCAN_USBBUS11            = TPCANHandle(0x50B) # PCAN-USB interface, channel 11
+PCAN_USBBUS12            = TPCANHandle(0x50C) # PCAN-USB interface, channel 12
+PCAN_USBBUS13            = TPCANHandle(0x50D) # PCAN-USB interface, channel 13
+PCAN_USBBUS14            = TPCANHandle(0x50E) # PCAN-USB interface, channel 14
+PCAN_USBBUS15            = TPCANHandle(0x50F) # PCAN-USB interface, channel 15
+PCAN_USBBUS16            = TPCANHandle(0x510) # PCAN-USB interface, channel 16
 
 PCAN_PCCBUS1             = TPCANHandle(0x61)  # PCAN-PC Card interface, channel 1
 PCAN_PCCBUS2             = TPCANHandle(0x62)  # PCAN-PC Card interface, channel 2
 
-PCAN_LANBUS1             = TPCANHandle(0x801)  # PCAN-LAN interface, channel 1
-PCAN_LANBUS2             = TPCANHandle(0x802)  # PCAN-LAN interface, channel 2
-PCAN_LANBUS3             = TPCANHandle(0x803)  # PCAN-LAN interface, channel 3
-PCAN_LANBUS4             = TPCANHandle(0x804)  # PCAN-LAN interface, channel 4
-PCAN_LANBUS5             = TPCANHandle(0x805)  # PCAN-LAN interface, channel 5
-PCAN_LANBUS6             = TPCANHandle(0x806)  # PCAN-LAN interface, channel 6
-PCAN_LANBUS7             = TPCANHandle(0x807)  # PCAN-LAN interface, channel 7
-PCAN_LANBUS8             = TPCANHandle(0x808)  # PCAN-LAN interface, channel 8
-PCAN_LANBUS9             = TPCANHandle(0x809)  # PCAN-LAN interface, channel 9
-PCAN_LANBUS10            = TPCANHandle(0x80A)  # PCAN-LAN interface, channel 10
-PCAN_LANBUS11            = TPCANHandle(0x80B)  # PCAN-LAN interface, channel 11
-PCAN_LANBUS12            = TPCANHandle(0x80C)  # PCAN-LAN interface, channel 12
-PCAN_LANBUS13            = TPCANHandle(0x80D)  # PCAN-LAN interface, channel 13
-PCAN_LANBUS14            = TPCANHandle(0x80E)  # PCAN-LAN interface, channel 14
-PCAN_LANBUS15            = TPCANHandle(0x80F)  # PCAN-LAN interface, channel 15
-PCAN_LANBUS16            = TPCANHandle(0x810)  # PCAN-LAN interface, channel 16
+PCAN_LANBUS1             = TPCANHandle(0x801) # PCAN-LAN interface, channel 1
+PCAN_LANBUS2             = TPCANHandle(0x802) # PCAN-LAN interface, channel 2
+PCAN_LANBUS3             = TPCANHandle(0x803) # PCAN-LAN interface, channel 3
+PCAN_LANBUS4             = TPCANHandle(0x804) # PCAN-LAN interface, channel 4
+PCAN_LANBUS5             = TPCANHandle(0x805) # PCAN-LAN interface, channel 5
+PCAN_LANBUS6             = TPCANHandle(0x806) # PCAN-LAN interface, channel 6
+PCAN_LANBUS7             = TPCANHandle(0x807) # PCAN-LAN interface, channel 7
+PCAN_LANBUS8             = TPCANHandle(0x808) # PCAN-LAN interface, channel 8
+PCAN_LANBUS9             = TPCANHandle(0x809) # PCAN-LAN interface, channel 9
+PCAN_LANBUS10            = TPCANHandle(0x80A) # PCAN-LAN interface, channel 10
+PCAN_LANBUS11            = TPCANHandle(0x80B) # PCAN-LAN interface, channel 11
+PCAN_LANBUS12            = TPCANHandle(0x80C) # PCAN-LAN interface, channel 12
+PCAN_LANBUS13            = TPCANHandle(0x80D) # PCAN-LAN interface, channel 13
+PCAN_LANBUS14            = TPCANHandle(0x80E) # PCAN-LAN interface, channel 14
+PCAN_LANBUS15            = TPCANHandle(0x80F) # PCAN-LAN interface, channel 15
+PCAN_LANBUS16            = TPCANHandle(0x810) # PCAN-LAN interface, channel 16
 
 # Represent the PCAN error and status codes 
-#
-PCAN_ERROR_OK            = TPCANStatus(0x00000)  # No error 
+PCAN_ERROR_OK            = TPCANStatus(0x00000)  # No error
 PCAN_ERROR_XMTFULL       = TPCANStatus(0x00001)  # Transmit buffer in CAN controller is full
 PCAN_ERROR_OVERRUN       = TPCANStatus(0x00002)  # CAN controller was read too late
 PCAN_ERROR_BUSLIGHT      = TPCANStatus(0x00004)  # Bus error: an error counter reached the 'light' limit
@@ -146,51 +139,59 @@ PCAN_ERROR_INITIALIZE    = TPCANStatus(0x4000000)# Channel is not initialized [V
 PCAN_ERROR_ILLOPERATION  = TPCANStatus(0x8000000)# Invalid operation [Value was changed from 0x80000 to 0x8000000]
 
 # PCAN devices
-#
-PCAN_NONE                = TPCANDevice(0x00)  # Undefined, unknown or not selected PCAN device value
-PCAN_PEAKCAN             = TPCANDevice(0x01)  # PCAN Non-Plug&Play devices. NOT USED WITHIN PCAN-Basic API
-PCAN_ISA                 = TPCANDevice(0x02)  # PCAN-ISA, PCAN-PC/104, and PCAN-PC/104-Plus
-PCAN_DNG                 = TPCANDevice(0x03)  # PCAN-Dongle
-PCAN_PCI                 = TPCANDevice(0x04)  # PCAN-PCI, PCAN-cPCI, PCAN-miniPCI, and PCAN-PCI Express
-PCAN_USB                 = TPCANDevice(0x05)  # PCAN-USB and PCAN-USB Pro
-PCAN_PCC                 = TPCANDevice(0x06)  # PCAN-PC Card
-PCAN_VIRTUAL             = TPCANDevice(0x07)  # PCAN Virtual hardware. NOT USED WITHIN PCAN-Basic API
-PCAN_LAN                 = TPCANDevice(0x08)  # PCAN Gateway devices
+PCAN_NONE                = TPCANDevice(0x00)     # Undefined, unknown or not selected PCAN device value
+PCAN_PEAKCAN             = TPCANDevice(0x01)     # PCAN Non-Plug&Play devices. NOT USED WITHIN PCAN-Basic API
+PCAN_ISA                 = TPCANDevice(0x02)     # PCAN-ISA, PCAN-PC/104, and PCAN-PC/104-Plus
+PCAN_DNG                 = TPCANDevice(0x03)     # PCAN-Dongle
+PCAN_PCI                 = TPCANDevice(0x04)     # PCAN-PCI, PCAN-cPCI, PCAN-miniPCI, and PCAN-PCI Express
+PCAN_USB                 = TPCANDevice(0x05)     # PCAN-USB and PCAN-USB Pro
+PCAN_PCC                 = TPCANDevice(0x06)     # PCAN-PC Card
+PCAN_VIRTUAL             = TPCANDevice(0x07)     # PCAN Virtual hardware. NOT USED WITHIN PCAN-Basic API
+PCAN_LAN                 = TPCANDevice(0x08)     # PCAN Gateway devices
 
 # PCAN parameters
-#
-PCAN_DEVICE_NUMBER       = TPCANParameter(0x01)  # PCAN-USB device number parameter
-PCAN_5VOLTS_POWER        = TPCANParameter(0x02)  # PCAN-PC Card 5-Volt power parameter
-PCAN_RECEIVE_EVENT       = TPCANParameter(0x03)  # PCAN receive event handler parameter
-PCAN_MESSAGE_FILTER      = TPCANParameter(0x04)  # PCAN message filter parameter
-PCAN_API_VERSION         = TPCANParameter(0x05)  # PCAN-Basic API version parameter
-PCAN_CHANNEL_VERSION     = TPCANParameter(0x06)  # PCAN device channel version parameter
-PCAN_BUSOFF_AUTORESET    = TPCANParameter(0x07)  # PCAN Reset-On-Busoff parameter
-PCAN_LISTEN_ONLY         = TPCANParameter(0x08)  # PCAN Listen-Only parameter
-PCAN_LOG_LOCATION        = TPCANParameter(0x09)  # Directory path for log files
-PCAN_LOG_STATUS          = TPCANParameter(0x0A)  # Debug-Log activation status
-PCAN_LOG_CONFIGURE       = TPCANParameter(0x0B)  # Configuration of the debugged information (LOG_FUNCTION_***)
-PCAN_LOG_TEXT            = TPCANParameter(0x0C)  # Custom insertion of text into the log file
-PCAN_CHANNEL_CONDITION   = TPCANParameter(0x0D)  # Availability status of a PCAN-Channel
-PCAN_HARDWARE_NAME       = TPCANParameter(0x0E)  # PCAN hardware name parameter
-PCAN_RECEIVE_STATUS      = TPCANParameter(0x0F)  # Message reception status of a PCAN-Channel
-PCAN_CONTROLLER_NUMBER   = TPCANParameter(0x10)  # CAN-Controller number of a PCAN-Channel
-PCAN_TRACE_LOCATION      = TPCANParameter(0x11)  # Directory path for PCAN trace files
-PCAN_TRACE_STATUS        = TPCANParameter(0x12)  # CAN tracing activation status
-PCAN_TRACE_SIZE          = TPCANParameter(0x13)  # Configuration of the maximum file size of a CAN trace
-PCAN_TRACE_CONFIGURE     = TPCANParameter(0x14)  # Configuration of the trace file storing mode (TRACE_FILE_***)
-PCAN_CHANNEL_IDENTIFYING = TPCANParameter(0x15)  # Physical identification of a USB based PCAN-Channel by blinking its associated LED
-PCAN_CHANNEL_FEATURES    = TPCANParameter(0x16)  # Capabilities of a PCAN device (FEATURE_***)
-PCAN_BITRATE_ADAPTING    = TPCANParameter(0x17)  # Using of an existing bit rate (PCAN-View connected to a channel)
-PCAN_BITRATE_INFO        = TPCANParameter(0x18)  # Configured bit rate as Btr0Btr1 value
-PCAN_BITRATE_INFO_FD     = TPCANParameter(0x19)  # Configured bit rate as TPCANBitrateFD string
-PCAN_BUSSPEED_NOMINAL    = TPCANParameter(0x1A)  # Configured nominal CAN Bus speed as Bits per seconds
-PCAN_BUSSPEED_DATA       = TPCANParameter(0x1B)  # Configured CAN data speed as Bits per seconds
-PCAN_IP_ADDRESS          = TPCANParameter(0x1C)  # Remote address of a LAN channel as string in IPv4 format
-PCAN_LAN_SERVICE_STATUS  = TPCANParameter(0x1D)  # Status of the Virtual PCAN-Gateway Service
+PCAN_DEVICE_NUMBER            = TPCANParameter(0x01)  # PCAN-USB device number parameter
+PCAN_5VOLTS_POWER             = TPCANParameter(0x02)  # PCAN-PC Card 5-Volt power parameter
+PCAN_RECEIVE_EVENT            = TPCANParameter(0x03)  # PCAN receive event handler parameter
+PCAN_MESSAGE_FILTER           = TPCANParameter(0x04)  # PCAN message filter parameter
+PCAN_API_VERSION              = TPCANParameter(0x05)  # PCAN-Basic API version parameter
+PCAN_CHANNEL_VERSION          = TPCANParameter(0x06)  # PCAN device channel version parameter
+PCAN_BUSOFF_AUTORESET         = TPCANParameter(0x07)  # PCAN Reset-On-Busoff parameter
+PCAN_LISTEN_ONLY              = TPCANParameter(0x08)  # PCAN Listen-Only parameter
+PCAN_LOG_LOCATION             = TPCANParameter(0x09)  # Directory path for log files
+PCAN_LOG_STATUS               = TPCANParameter(0x0A)  # Debug-Log activation status
+PCAN_LOG_CONFIGURE            = TPCANParameter(0x0B)  # Configuration of the debugged information (LOG_FUNCTION_***)
+PCAN_LOG_TEXT                 = TPCANParameter(0x0C)  # Custom insertion of text into the log file
+PCAN_CHANNEL_CONDITION        = TPCANParameter(0x0D)  # Availability status of a PCAN-Channel
+PCAN_HARDWARE_NAME            = TPCANParameter(0x0E)  # PCAN hardware name parameter
+PCAN_RECEIVE_STATUS           = TPCANParameter(0x0F)  # Message reception status of a PCAN-Channel
+PCAN_CONTROLLER_NUMBER        = TPCANParameter(0x10)  # CAN-Controller number of a PCAN-Channel
+PCAN_TRACE_LOCATION           = TPCANParameter(0x11)  # Directory path for PCAN trace files
+PCAN_TRACE_STATUS             = TPCANParameter(0x12)  # CAN tracing activation status
+PCAN_TRACE_SIZE               = TPCANParameter(0x13)  # Configuration of the maximum file size of a CAN trace
+PCAN_TRACE_CONFIGURE          = TPCANParameter(0x14)  # Configuration of the trace file storing mode (TRACE_FILE_***)
+PCAN_CHANNEL_IDENTIFYING      = TPCANParameter(0x15)  # Physical identification of a USB based PCAN-Channel by blinking its associated LED
+PCAN_CHANNEL_FEATURES         = TPCANParameter(0x16)  # Capabilities of a PCAN device (FEATURE_***)
+PCAN_BITRATE_ADAPTING         = TPCANParameter(0x17)  # Using of an existing bit rate (PCAN-View connected to a channel)
+PCAN_BITRATE_INFO             = TPCANParameter(0x18)  # Configured bit rate as Btr0Btr1 value
+PCAN_BITRATE_INFO_FD          = TPCANParameter(0x19)  # Configured bit rate as TPCANBitrateFD string
+PCAN_BUSSPEED_NOMINAL         = TPCANParameter(0x1A)  # Configured nominal CAN Bus speed as Bits per seconds
+PCAN_BUSSPEED_DATA            = TPCANParameter(0x1B)  # Configured CAN data speed as Bits per seconds
+PCAN_IP_ADDRESS               = TPCANParameter(0x1C)  # Remote address of a LAN channel as string in IPv4 format
+PCAN_LAN_SERVICE_STATUS       = TPCANParameter(0x1D)  # Status of the Virtual PCAN-Gateway Service
+PCAN_ALLOW_STATUS_FRAMES      = TPCANParameter(0x1E)  # Status messages reception status within a PCAN-Channel
+PCAN_ALLOW_RTR_FRAMES         = TPCANParameter(0x1F)  # RTR messages reception status within a PCAN-Channel
+PCAN_ALLOW_ERROR_FRAMES       = TPCANParameter(0x20)  # Error messages reception status within a PCAN-Channel
+PCAN_INTERFRAME_DELAY         = TPCANParameter(0x21)  # Delay, in microseconds, between sending frames
+PCAN_ACCEPTANCE_FILTER_11BIT  = TPCANParameter(0x22)  # Filter over code and mask patterns for 11-Bit messages
+PCAN_ACCEPTANCE_FILTER_29BIT  = TPCANParameter(0x23)  # Filter over code and mask patterns for 29-Bit messages
+PCAN_IO_DIGITAL_CONFIGURATION = TPCANParameter(0x24)  # Output mode of 32 digital I/O pin of a PCAN-USB Chip. 1: Output-Active 0 : Output Inactive
+PCAN_IO_DIGITAL_VALUE         = TPCANParameter(0x25)  # Value assigned to a 32 digital I/O pins of a PCAN-USB Chip
+PCAN_IO_DIGITAL_SET           = TPCANParameter(0x26)  # Value assigned to a 32 digital I/O pins of a PCAN-USB Chip - Multiple digital I/O pins to 1 = High
+PCAN_IO_DIGITAL_CLEAR         = TPCANParameter(0x27)  # Clear multiple digital I/O pins to 0
+PCAN_IO_ANALOG_VALUE          = TPCANParameter(0x28)  # Get value of a single analog input pin
 
 # PCAN parameter values
-#
 PCAN_PARAMETER_OFF       = int(0x00)  # The PCAN parameter is not set (inactive)
 PCAN_PARAMETER_ON        = int(0x01)  # The PCAN parameter is set (active)
 PCAN_FILTER_CLOSE        = int(0x00)  # The PCAN filter is closed. No messages will be received
@@ -201,13 +202,13 @@ PCAN_CHANNEL_AVAILABLE   = int(0x01)  # The PCAN-Channel handle is available to
 PCAN_CHANNEL_OCCUPIED    = int(0x02)  # The PCAN-Channel handle is valid, and is already being used
 PCAN_CHANNEL_PCANVIEW    = PCAN_CHANNEL_AVAILABLE | PCAN_CHANNEL_OCCUPIED  # The PCAN-Channel handle is already being used by a PCAN-View application, but is available to connect
 
-LOG_FUNCTION_DEFAULT     = int(0x00)   # Logs system exceptions / errors
-LOG_FUNCTION_ENTRY       = int(0x01)   # Logs the entries to the PCAN-Basic API functions 
-LOG_FUNCTION_PARAMETERS  = int(0x02)   # Logs the parameters passed to the PCAN-Basic API functions 
-LOG_FUNCTION_LEAVE       = int(0x04)   # Logs the exits from the PCAN-Basic API functions 
-LOG_FUNCTION_WRITE       = int(0x08)   # Logs the CAN messages passed to the CAN_Write function
-LOG_FUNCTION_READ        = int(0x10)   # Logs the CAN messages received within the CAN_Read function
-LOG_FUNCTION_ALL         = int(0xFFFF) # Logs all possible information within the PCAN-Basic API functions
+LOG_FUNCTION_DEFAULT     = int(0x00)  # Logs system exceptions / errors
+LOG_FUNCTION_ENTRY       = int(0x01)  # Logs the entries to the PCAN-Basic API functions
+LOG_FUNCTION_PARAMETERS  = int(0x02)  # Logs the parameters passed to the PCAN-Basic API functions
+LOG_FUNCTION_LEAVE       = int(0x04)  # Logs the exits from the PCAN-Basic API functions
+LOG_FUNCTION_WRITE       = int(0x08)  # Logs the CAN messages passed to the CAN_Write function
+LOG_FUNCTION_READ        = int(0x10)  # Logs the CAN messages received within the CAN_Read function
+LOG_FUNCTION_ALL         = int(0xFFFF)# Logs all possible information within the PCAN-Basic API functions
 
 TRACE_FILE_SINGLE        = int(0x00)  # A single file is written until it size reaches PAN_TRACE_SIZE
 TRACE_FILE_SEGMENTED     = int(0x01)  # Traced data is distributed in several files with size PAN_TRACE_SIZE
@@ -216,54 +217,54 @@ TRACE_FILE_TIME          = int(0x04)  # Includes the start time into the name of
 TRACE_FILE_OVERWRITE     = int(0x80)  # Causes the overwriting of available traces (same name)
 
 FEATURE_FD_CAPABLE       = int(0x01)  # Device supports flexible data-rate (CAN-FD)
+FEATURE_DELAY_CAPABLE    = int(0x02)  # Device supports a delay between sending frames (FPGA based USB devices)
+FEATURE_IO_CAPABLE       = int(0x04)  # Device supports I/O functionality for electronic circuits (USB-Chip devices)
 
 SERVICE_STATUS_STOPPED   = int(0x01)  # The service is not running
 SERVICE_STATUS_RUNNING   = int(0x04)  # The service is running
 
 # PCAN message types
-#
 PCAN_MESSAGE_STANDARD    = TPCANMessageType(0x00)  # The PCAN message is a CAN Standard Frame (11-bit identifier)
 PCAN_MESSAGE_RTR         = TPCANMessageType(0x01)  # The PCAN message is a CAN Remote-Transfer-Request Frame
 PCAN_MESSAGE_EXTENDED    = TPCANMessageType(0x02)  # The PCAN message is a CAN Extended Frame (29-bit identifier)
 PCAN_MESSAGE_FD          = TPCANMessageType(0x04)  # The PCAN message represents a FD frame in terms of CiA Specs
 PCAN_MESSAGE_BRS         = TPCANMessageType(0x08)  # The PCAN message represents a FD bit rate switch (CAN data at a higher bit rate)
 PCAN_MESSAGE_ESI         = TPCANMessageType(0x10)  # The PCAN message represents a FD error state indicator(CAN FD transmitter was error active)
+PCAN_MESSAGE_ERRFRAME    = TPCANMessageType(0x40)  # The PCAN message represents an error frame
 PCAN_MESSAGE_STATUS      = TPCANMessageType(0x80)  # The PCAN message represents a PCAN status message
 
 # Frame Type / Initialization Mode
-#
-PCAN_MODE_STANDARD       = PCAN_MESSAGE_STANDARD  
-PCAN_MODE_EXTENDED       = PCAN_MESSAGE_EXTENDED  
+PCAN_MODE_STANDARD       = PCAN_MESSAGE_STANDARD
+PCAN_MODE_EXTENDED       = PCAN_MESSAGE_EXTENDED
 
 # Baud rate codes = BTR0/BTR1 register values for the CAN controller.
 # You can define your own Baud rate with the BTROBTR1 register.
 # Take a look at www.peak-system.com for our free software "BAUDTOOL" 
 # to calculate the BTROBTR1 register for every bit rate and sample point.
-#
-PCAN_BAUD_1M             = TPCANBaudrate(0x0014) #   1 MBit/s
-PCAN_BAUD_800K           = TPCANBaudrate(0x0016) # 800 kBit/s
-PCAN_BAUD_500K           = TPCANBaudrate(0x001C) # 500 kBit/s
-PCAN_BAUD_250K           = TPCANBaudrate(0x011C) # 250 kBit/s
-PCAN_BAUD_125K           = TPCANBaudrate(0x031C) # 125 kBit/s
-PCAN_BAUD_100K           = TPCANBaudrate(0x432F) # 100 kBit/s
+
+PCAN_BAUD_1M             = TPCANBaudrate(0x0014) #   1     MBit/s
+PCAN_BAUD_800K           = TPCANBaudrate(0x0016) # 800     kBit/s
+PCAN_BAUD_500K           = TPCANBaudrate(0x001C) # 500     kBit/s
+PCAN_BAUD_250K           = TPCANBaudrate(0x011C) # 250     kBit/s
+PCAN_BAUD_125K           = TPCANBaudrate(0x031C) # 125     kBit/s
+PCAN_BAUD_100K           = TPCANBaudrate(0x432F) # 100     kBit/s
 PCAN_BAUD_95K            = TPCANBaudrate(0xC34E) #  95,238 kBit/s
 PCAN_BAUD_83K            = TPCANBaudrate(0x852B) #  83,333 kBit/s
-PCAN_BAUD_50K            = TPCANBaudrate(0x472F) #  50 kBit/s
+PCAN_BAUD_50K            = TPCANBaudrate(0x472F) #  50     kBit/s
 PCAN_BAUD_47K            = TPCANBaudrate(0x1414) #  47,619 kBit/s
 PCAN_BAUD_33K            = TPCANBaudrate(0x8B2F) #  33,333 kBit/s
-PCAN_BAUD_20K            = TPCANBaudrate(0x532F) #  20 kBit/s
-PCAN_BAUD_10K            = TPCANBaudrate(0x672F) #  10 kBit/s
-PCAN_BAUD_5K             = TPCANBaudrate(0x7F7F) #   5 kBit/s
+PCAN_BAUD_20K            = TPCANBaudrate(0x532F) #  20     kBit/s
+PCAN_BAUD_10K            = TPCANBaudrate(0x672F) #  10     kBit/s
+PCAN_BAUD_5K             = TPCANBaudrate(0x7F7F) #   5     kBit/s
 
 # Represents the configuration for a CAN bit rate
-# Note: 
+# Note:
 #    * Each parameter and its value must be separated with a '='.
 #    * Each pair of parameter/value must be separated using ','. 
 #
 # Example:
-#    f_clock=80000000,nom_brp=0,nom_tseg1=13,nom_tseg2=0,nom_sjw=0,data_brp=0,data_tseg1=13,data_tseg2=0,data_sjw=0
+#    f_clock=80000000,nom_brp=10,nom_tseg1=5,nom_tseg2=2,nom_sjw=1,data_brp=4,data_tseg1=7,data_tseg2=2,data_sjw=1
 #
-# Patched for python-can: use bytes to fix incompatibility with Python 3
 PCAN_BR_CLOCK            = TPCANBitrateFD(b"f_clock")
 PCAN_BR_CLOCK_MHZ        = TPCANBitrateFD(b"f_clock_mhz")
 PCAN_BR_NOM_BRP          = TPCANBitrateFD(b"nom_brp")
@@ -278,45 +279,40 @@ PCAN_BR_DATA_SJW         = TPCANBitrateFD(b"data_sjw")
 PCAN_BR_DATA_SAMPLE      = TPCANBitrateFD(b"data_ssp_offset")
 
 # Supported No-Plug-And-Play Hardware types
-#
 PCAN_TYPE_ISA            = TPCANType(0x01)  # PCAN-ISA 82C200
 PCAN_TYPE_ISA_SJA        = TPCANType(0x09)  # PCAN-ISA SJA1000
-PCAN_TYPE_ISA_PHYTEC     = TPCANType(0x04)  # PHYTEC ISA 
+PCAN_TYPE_ISA_PHYTEC     = TPCANType(0x04)  # PHYTEC ISA
 PCAN_TYPE_DNG            = TPCANType(0x02)  # PCAN-Dongle 82C200
 PCAN_TYPE_DNG_EPP        = TPCANType(0x03)  # PCAN-Dongle EPP 82C200
 PCAN_TYPE_DNG_SJA        = TPCANType(0x05)  # PCAN-Dongle SJA1000
 PCAN_TYPE_DNG_SJA_EPP    = TPCANType(0x06)  # PCAN-Dongle EPP SJA1000
 
-# Represents a PCAN message
-#
+
 class TPCANMsg (Structure):
     """
     Represents a PCAN message
     """
-    _fields_ = [ ("ID",      c_uint),          # 11/29-bit message identifier
+    _fields_ = [ ("ID",      c_uint),           # 11/29-bit message identifier
                  ("MSGTYPE", TPCANMessageType), # Type of the message
                  ("LEN",     c_ubyte),          # Data Length Code of the message (0..8)
                  ("DATA",    c_ubyte * 8) ]     # Data of the message (DATA[0]..DATA[7])
 
-# Represents a timestamp of a received PCAN message
-# Total Microseconds = micros + 1000 * millis + 0x100000000 * 1000 * millis_overflow
-#
+
 class TPCANTimestamp (Structure):
     """
     Represents a timestamp of a received PCAN message
     Total Microseconds = micros + 1000 * millis + 0x100000000 * 1000 * millis_overflow
     """
-    _fields_ = [ ("millis", c_uint),           # Base-value: milliseconds: 0.. 2^32-1
+    _fields_ = [ ("millis", c_uint),            # Base-value: milliseconds: 0.. 2^32-1
                  ("millis_overflow", c_ushort), # Roll-arounds of millis
                  ("micros", c_ushort) ]         # Microseconds: 0..999
 
-# Represents a PCAN message from a FD capable hardware
-#
+
 class TPCANMsgFD (Structure):
     """
     Represents a PCAN message
     """
-    _fields_ = [ ("ID",      c_uint),          # 11/29-bit message identifier
+    _fields_ = [ ("ID",      c_uint),           # 11/29-bit message identifier
                  ("MSGTYPE", TPCANMessageType), # Type of the message
                  ("DLC",     c_ubyte),          # Data Length Code of the message (0..15)
                  ("DATA",    c_ubyte * 64) ]    # Data of the message (DATA[0]..DATA[63])
@@ -325,15 +321,12 @@ class TPCANMsgFD (Structure):
 # PCAN-Basic API function declarations
 #///////////////////////////////////////////////////////////
 
-# PCAN-Basic API class implementation
-#
 class PCANBasic:
+    """PCAN-Basic API class implementation
     """
-      PCAN-Basic API class implementation
-    """      
+
     def __init__(self):
         # Loads the PCANBasic.dll
-        #     
         if platform.system() == 'Windows':
             self.__m_dllBasic = windll.LoadLibrary("PCANBasic")
         else:
@@ -341,16 +334,14 @@ class PCANBasic:
         if self.__m_dllBasic == None:
             logger.error("Exception: The PCAN-Basic DLL couldn't be loaded!")
 
-    # Initializes a PCAN Channel
-    #
     def Initialize(
         self,
-        Channel,   
-        Btr0Btr1,  
-        HwType = TPCANType(0),  
+        Channel,
+        Btr0Btr1,
+        HwType = TPCANType(0),
         IOPort = c_uint(0),
         Interrupt = c_ushort(0)):
-        
+
         """
           Initializes a PCAN Channel
 
@@ -370,14 +361,12 @@ class PCANBasic:
         except:
             logger.error("Exception on PCANBasic.Initialize")
             raise
-        
-    # Initializes a FD capable PCAN Channel  
-    #
+
     def InitializeFD(
         self,
-        Channel,   
+        Channel,
         BitrateFD):
-        
+
         """
           Initializes a FD capable PCAN Channel  
 
@@ -393,8 +382,8 @@ class PCANBasic:
             nom_brp, nom_sjw, nom_tseg1, nom_tseg2.
           * Following Parameters are optional (not used yet): data_ssp_offset, nom_samp
 
-    Example:
-          f_clock_mhz=80,nom_brp=0,nom_tseg1=13,nom_tseg2=0,nom_sjw=0,data_brp=0,data_tseg1=13,data_tseg2=0,data_sjw=0
+        Example:
+          f_clock=80000000,nom_brp=10,nom_tseg1=5,nom_tseg2=2,nom_sjw=1,data_brp=4,data_tseg1=7,data_tseg2=2,data_sjw=1
 
         Returns:
           A TPCANStatus error code
@@ -406,8 +395,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.InitializeFD")
             raise
 
-    #  Uninitializes one or all PCAN Channels initialized by CAN_Initialize
-    #
     def Uninitialize(
         self,
         Channel):
@@ -431,8 +418,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.Uninitialize")
             raise
 
-    #  Resets the receive and transmit queues of the PCAN Channel
-    #
     def Reset(
         self,
         Channel):
@@ -455,9 +440,7 @@ class PCANBasic:
         except:
             logger.error("Exception on PCANBasic.Reset")
             raise
-            
-    #  Gets the current status of a PCAN Channel
-    #
+
     def GetStatus(
         self,
         Channel):
@@ -478,8 +461,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.GetStatus")
             raise
 
-    # Reads a CAN message from the receive queue of a PCAN Channel
-    #
     def Read(
         self,
         Channel):
@@ -508,10 +489,8 @@ class PCANBasic:
             return TPCANStatus(res),msg,timestamp
         except:
             logger.error("Exception on PCANBasic.Read")
-            raise           
+            raise
 
-    # Reads a CAN message from the receive queue of a FD capable PCAN Channel
-    #
     def ReadFD(
         self,
         Channel):
@@ -540,10 +519,8 @@ class PCANBasic:
             return TPCANStatus(res),msg,timestamp
         except:
             logger.error("Exception on PCANBasic.ReadFD")
-            raise  
+            raise
 
-    # Transmits a CAN message 
-    #
     def Write(
         self,
         Channel,
@@ -566,8 +543,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.Write")
             raise
 
-    # Transmits a CAN message over a FD capable PCAN Channel
-    #
     def WriteFD(
         self,
         Channel,
@@ -590,8 +565,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.WriteFD")
             raise
 
-    # Configures the reception filter 
-    #
     def FilterMessages(
         self,
         Channel,
@@ -623,8 +596,6 @@ class PCANBasic:
             logger.error("Exception on PCANBasic.FilterMessages")
             raise
 
-    # Retrieves a PCAN Channel value 
-    #
     def GetValue(
         self,
         Channel,
@@ -648,7 +619,7 @@ class PCANBasic:
         
         Returns:
           A touple with 2 values
-        """        
+        """
         try:
             if Parameter == PCAN_API_VERSION or Parameter == PCAN_HARDWARE_NAME or Parameter == PCAN_CHANNEL_VERSION or Parameter == PCAN_LOG_LOCATION or Parameter == PCAN_TRACE_LOCATION or Parameter == PCAN_BITRATE_INFO_FD or Parameter == PCAN_IP_ADDRESS:
                 mybuffer = create_string_buffer(256)
@@ -659,11 +630,8 @@ class PCANBasic:
             return TPCANStatus(res),mybuffer.value
         except:
             logger.error("Exception on PCANBasic.GetValue")
-            raise            
+            raise
 
-    # Returns a descriptive text of a given TPCANStatus
-    # error code, in any desired language
-    #
     def SetValue(
         self,
         Channel,
@@ -687,7 +655,7 @@ class PCANBasic:
         
         Returns:
           A TPCANStatus error code
-        """        
+        """
         try:
             if Parameter == PCAN_LOG_LOCATION or Parameter == PCAN_LOG_TEXT or Parameter == PCAN_TRACE_LOCATION:
                 mybuffer = create_string_buffer(256)
@@ -715,7 +683,7 @@ class PCANBasic:
           Neutral (0x00), German (0x07), English (0x09), Spanish (0x0A),
           Italian (0x10) and French (0x0C)          
 
-          The return value of this method is a 2-touple, where 
+          The return value of this method is a 2-touple, where
           the first value is the result (TPCANStatus) of the method and
           the second one, the error text
           
@@ -725,11 +693,11 @@ class PCANBasic:
         
         Returns:
           A touple with 2 values
-        """  
+        """
         try:
             mybuffer = create_string_buffer(256)
             res = self.__m_dllBasic.CAN_GetErrorText(Error,Language,byref(mybuffer))
             return TPCANStatus(res),mybuffer.value
         except:
             logger.error("Exception on PCANBasic.GetErrorText")
-            raise            
+            raise
