@@ -1,9 +1,20 @@
+#!/usr/bin/env python
+# coding: utf-8
+
+"""
+See the :class:`Logger` class.
+"""
+
+import logging
+
 from .asc import ASCWriter
 from .blf import BLFWriter
+from .canutils import CanutilsLogWriter
 from .csv import CSVWriter
 from .sqlite import SqliteWriter
 from .stdout import Printer
-from .log import CanutilsLogWriter
+
+log = logging.getLogger("can.io.logger")
 
 
 class Logger(object):
@@ -15,6 +26,7 @@ class Logger(object):
       * .blf :class:`can.BLFWriter`
       * .csv: :class:`can.CSVWriter`
       * .db: :class:`can.SqliteWriter`
+      * .log :class:`can.CanutilsLogWriter`
       * other: :class:`can.Printer`
 
     Note this class itself is just a dispatcher,
@@ -22,19 +34,20 @@ class Logger(object):
     be created when instantiating this class.
     """
 
-    @classmethod
-    def __new__(cls, other, filename):
+    @staticmethod
+    def __new__(cls, filename):
         if not filename:
             return Printer()
         elif filename.endswith(".asc"):
             return ASCWriter(filename)
-        elif filename.endswith(".log"):
-            return CanutilsLogWriter(filename)
         elif filename.endswith(".blf"):
             return BLFWriter(filename)
         elif filename.endswith(".csv"):
             return CSVWriter(filename)
         elif filename.endswith(".db"):
             return SqliteWriter(filename)
+        elif filename.endswith(".log"):
+            return CanutilsLogWriter(filename)
         else:
+            log.info('unknown file type "%s", falling pack to can.Printer', filename)
             return Printer(filename)
